# -*- coding: utf-8 -*-

#  PycaWM
#  stickyrootedges.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('stickyrootedges module needs at least Python 2.5!')

from Xlib import X, display

from pycawm.hookmanager import (add_pre_hook, add_post_hook,
                                add_instance_pre_hook,
                                remove_pre_hook, remove_post_hook,
                                remove_instance_pre_hook)
from pycawm import Client

class StickyRootEdges(object):
    def __init__(self, wm, threshold=20):
        self.wm = wm
        self.threshold = threshold
        self.pos_x = None
        self.pos_y = None
        self.moved_client = None
        add_post_hook(wm.start_moving, self.start_moving)
        add_pre_hook(wm.button_release_reply, self.end_moving)

    def destroy(self):
        remove_post_hook(self.wm.start_moving, self.start_moving)
        remove_pre_hook(self.wm.button_release_reply, self.end_moving)
        if self.moved_client is not None:
            self.end_moving(force=True)

    def start_moving(self, wm, client):
        self.pos_x = None
        self.pos_y = None
        self.moved_client = client
        add_instance_pre_hook(client.move, self.root_window_edges_sticky)

    def end_moving(self, wm=None, event=None, force=False):
        if force or self.wm.state == self.wm.moving:
            remove_instance_pre_hook(self.moved_client.move,
                                     self.root_window_edges_sticky)
            self.pos_x = None
            self.pos_y = None
            self.moved_client = None

    def root_window_edges_sticky(self, client, coords):
        geo = client.geometry
        screen = client.screen

        if self.pos_x is None:
            if (coords['x'] + geo.width >
                screen.width_in_pixels - self.threshold or
                coords['x'] < self.threshold):
                self.pos_x = coords['x']
        else:
            self.pos_x += coords['x'] - geo.x

        if self.pos_y is None:
            if (coords['y'] + geo.height >
                screen.height_in_pixels - self.threshold or
                coords['y'] < self.threshold):
                self.pos_y = coords['y']
        else:
            self.pos_y += coords['y'] - geo.y

        if (self.pos_x is not None and geo.x == 0 or
            geo.x == screen.width_in_pixels - geo.width):
            coords['x'] = self.pos_x

        if (self.pos_y is not None and geo.y == 0 or
            geo.y == screen.height_in_pixels - geo.height):
            coords['y'] = self.pos_y

        if (coords['x'] + geo.width >
            screen.width_in_pixels - self.threshold):
            coords['x'] = screen.width_in_pixels - geo.width
        if coords['x'] < self.threshold:
            coords['x'] = 0
        if coords['y'] + geo.height > screen.height_in_pixels - self.threshold:
            coords['y'] = screen.height_in_pixels - geo.height
        if coords['y'] < self.threshold:
            coords['y'] = 0

        if (coords['x'] != 0 and
            coords['x'] != screen.width_in_pixels - geo.width):
            self.pos_x = None

        if (coords['y'] != 0 and
            coords['y'] != screen.height_in_pixels - geo.height):
            self.pos_y = None
